﻿/////////////////////////////////////////////////////////////////////////////////////////////////
//
// Flash Media Server Sample Publish & Subscribe Demonstration App for LiveStreamCast 1.0
// Copyright 2009 Adobe Systems Incorporated
// All Rights Reserved.
//
// NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the 
// terms of the Adobe license agreement accompanying it.  If you have received this file from a 
// source other than Adobe, then your use, modification, or distribution of it requires the prior 
// written permission of Adobe.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

// connection & stream name text variables
var fms_pub_url:String = "rtmp://localhost/livestreamcast_origin";
var fms_sub_url:String = "rtmp://localhost/livestreamcast_edge";

var streamName:String = "livestream1";

// set textfields
fms_pub_url_txt.text = fms_pub_url;
fms_sub_url_txt.text = fms_sub_url;
fms_streamname.text = streamName;

var pubDebugState:Boolean = false;
var subDebugState:Boolean = false;

//////////////////////////////////////////////////////
// Publisher NetConnection object
var publish_nc:NetConnection = new NetConnection();
//////////////////////////////////////////////////////
// Subscriber NetConnection object
var subscribe_nc:NetConnection = new NetConnection();

// publish & subscribe NetConnection & Netstream objects
var publish_ns:NetStream;
var subscribe_ns:NetStream;

//debug text area
var debug_ta:mx.controls.TextArea;

//publish type
st_publishType = "live";

//audio sample rate array
var sampleRate_array:Array = new Array(5, 8, 11, 16, 22, 44);
//set audio codec
var kAudioNM:Number = 0;
var kAudioSpeex:Number = 1;
var codecType:Number = kAudioNM;

//get Cam & set quality
var client_cam = Camera.get();
client_cam.setQuality(0,100);

//get Mic - default to Nellymoser 8K
var client_mic = Microphone.get();

//attach Cam to Live Video stage object
Publish_Video.attachVideo(client_cam);

///////////////////////////////////////
// init Event listeners
var connectPubBtnListener:Object = new Object();
var connectSubBtnListener:Object = new Object();
var pubBtnListener:Object = new Object();
var subBtnListener:Object = new Object();
var camBtnListener:Object = new Object();
var nsBufPubListener:Object = new Object();
var nsBufSubListener:Object = new Object();
var clearDBListener:Object = new Object();
var sampleRateListener:Object = new Object();
var audioCodecListener:Object = new Object();
var pubDebugListener:Object = new Object();
var subDebugListener:Object = new Object();

///////////////////////////////////////
// set the button event listeners to functions
Connect_Pub_btn.addEventListener("click", connectPubBtnListener);
Connect_Sub_btn.addEventListener("click", connectSubBtnListener);
Publish_btn.addEventListener("click", pubBtnListener);
Subscribe_btn.addEventListener("click", subBtnListener);
SetCamera_btn.addEventListener("click", camBtnListener);
nsBufPub_btn.addEventListener("click", nsBufPubListener);
nsBufSub_btn.addEventListener("click", nsBufSubListener);
clearDebug_btn.addEventListener("click", clearDBListener);
pubDebug_cb.addEventListener("click", pubDebugListener);
subDebug_cb.addEventListener("click", subDebugListener);
///////////////////////////////////////
// audio codec
rad_codec_nellymoser.addEventListener('click', audioCodecListener);
rad_codec_speex.addEventListener('click', audioCodecListener);

//////////////////////////////////////
//audio samplerate radio buttons
rad_44.addEventListener('click', sampleRateListener);
rad_22.addEventListener('click', sampleRateListener);
rad_16.addEventListener('click', sampleRateListener);
rad_11.addEventListener('click', sampleRateListener);
rad_8.addEventListener('click', sampleRateListener);
rad_5.addEventListener('click', sampleRateListener);

pubDebugListener.click = function(eventObj:Object)
{
	pubDebugState = pubDebug_cb.selected;
}

subDebugListener.click = function(eventObj:Object)
{
	subDebugState = subDebug_cb.selected;
}


///////////////////////////////////////////////////////
// Connect to Publishing OriginNode Server
connectPubBtnListener.click = function(eventObj:Object)
{
	debugtxt("[Connect Publisher]");
	connectPubClicked();
}
///////////////////////////////////////////////////////
// Publish stream
pubBtnListener.click = function(eventObj:Object)
{
	publishClicked();
}

///////////////////////////////////////////////////////
// Connect to Subscribing EdgeNode Server
connectSubBtnListener.click = function(eventObj:Object)
{
	debugtxt("[Connect Subscriber]");
	connectSubClicked();
}

///////////////////////////////////////////////////////
// Subscribe to stream
subBtnListener.click = function(eventObj:Object) 
{
	subscriberClicked();
}

///////////////////////////////////////////////////////
//set Publish Netstream buffer time
nsBufPubListener.click = function(eventObj:Object) 
{
	debugtxt("[SetPub NS]");
	
	var nsBufPubr:Number = ns_bufPub.text;
	debugtxt("pubNSBuffer= " + nsBufPubr);
	publish_ns.setBufferTime(nsBufPubr);
}

///////////////////////////////////////////////////////
//set Subscribe Netstream buffer time
nsBufSubListener.click = function(eventObj:Object) 
{
	debugtxt("[setSub NS]");
	
	var nsBufPubr2:Number = ns_bufSub.text;
	debugtxt("subNSBuffer= " + nsBufPubr2);
	subscribe_ns.setBufferTime(nsBufPubr2);
}

///////////////////////////////////////////////////////
//clear debug text area
clearDBListener.click = function(eventObj:Object)
{
	debug_ta.text = " ";
	debug_ta.vPosition = debug_ta.maxVPosition
}

/////////////////////////////////////////////////
// NetConnect Publisher Button
function connectPubClicked()
{
	if(Connect_Pub_btn.getLabel() == "Connect"){
		
		publish_nc.connect(fms_pub_url_txt.text);
		
	}else{
		closePublish(true);
		publish_nc.close();
	
		Connect_Pub_btn.setLabel("Connect");
	}
}

/////////////////////////////////////////////////
// Publish Button
function publishClicked()
{
	if(Publish_btn.getLabel() == "Publish"){
		Publish_btn.setLabel("Stop");
		Publish_btn.setStyle("themeColor", "0xFF0000");
		debugtxt("***publishing*** = " + fms_streamname.text);
		
		initPubStream(publish_nc);

	}else{
		
		closePublish(false);
	}
}

//////////////////////////////////////////////////////
// init publisher stream
function initPubStream(this_nc:NetConnection)
{
	debugtxt("initPubStream");
	
	publish_nc.call("FCPublish", null, fms_streamname.text);
	
}

//////////////////////////////////////////////////////
// close Publish stream & reset button
function closePublish(disableBtn:Boolean):Void
{
	debugtxt("closePublish");
	
	if(publish_ns){
		publish_ns.call("FCUnpublish", null, streamName);
		publish_ns.publish(false);
		publish_ns.close();
		publish_ns = null;
	}
	
	Publish_btn.setLabel("Publish");
	Publish_btn.setStyle("themeColor", "0xcccccc");
	if(disableBtn == true)
		Publish_btn.enabled = false;
}

//////////////////////////////////////////////////////
// Get Connection Success status
publish_nc.onStatus = function(ncInfoObj:Object):Void  {
	
	if(pubDebugState)
		debugtxt(ncInfoObj.code);
	
	if (ncInfoObj.code == "NetConnection.Connect.Success") {
		Connect_Pub_btn.setLabel("Close");
		Publish_btn.enabled = true;
	}
};

////////////////////////////////////////////////////
// FCPublish callback
publish_nc.onFCPublish = function(info)
{
	debugtxt("*** [onFCPublish] ***");
	
	if(pubDebugState)
		debugtxt("info: " + info.code);
	if (info.code == "NetStream.Publish.Start")
	{
		publish_ns = new NetStream(publish_nc);
		
		//always a good practice to set the NS buffer (seconds)
		publish_ns.setBufferTime(1);
		
		var debug1_interval:Number = setInterval(debug1Thing, 100, publish_ns);
		
		publish_ns.attachVideo(client_cam);
		publish_ns.attachAudio(client_mic);
		publish_ns.publish(fms_streamname.text);
		publish_ns.onStatus = function( info ){
			if(pubDebugState){
				debugtxt (info.code);
				debugtxt (info.description);
			}
		}
	}
	else if (info.code == "NetStream.Publish.BadName")
	{
		if(pubDebugState)
			debugtxt (info.code + ", " + info.description);
	}
}

////////////////////////////////////////////////////
// FCUnPublish callback
publish_nc.onFCUnpublish = function(info)
{
	debugtxt("[*** onFCUnpublish ***]");
			 
}

/////////////////////////////////////////////////
// NetConnect Subscriber Button
function connectSubClicked()
{
	if(Connect_Sub_btn.getLabel() == "Connect"){
		
		subscribe_nc.connect(fms_sub_url_txt.text);
		
	}else{
		closeSubscribe(true);
		subscribe_nc.close();
		
		Connect_Sub_btn.setLabel("Connect");
	}
}

////////////////////////////////////////////////////
// Subscribe Button
function subscriberClicked()
{
	debugtxt("[Subscribe clicked] = " + streamName);
	
	if(Subscribe_btn.getLabel() == "Subscribe"){
		Subscribe_btn.setLabel("Stop");
		Subscribe_btn.setStyle("themeColor", "0x00ff00");
		
		initSubStream(subscribe_nc);
		
	}else{
		Subscribe_btn.setLabel("Subscribe");
		Subscribe_btn.setStyle("themeColor", "0xcccccc");
		
		closeSubscribe(false);
	}
}

//////////////////////////////////////////////////////
// init subscriber stream
function initSubStream(this_nc:NetConnection)
{
	debugtxt("initSubStream");
	streamName = fms_streamname.text;
	
	subscribe_nc.call("FCSubscribe", null, streamName);
	
}

//////////////////////////////////////////////////////
// close Subscribe stream & reset button
function closeSubscribe(disableBtn:Boolean):Void
{
	debugtxt("closeSubscribe");
	
	if(subscribe_ns){
		subscribe_nc.call("FCUnsubscribe", null, streamName);
		subscribe_ns.close();
		subscribe_ns = null;
	}
	Subscribe_btn.setLabel("Subscribe");
	if(disableBtn == true){
		Subscribe_btn.enabled = false;
	}
}

//////////////////////////////////////////////////////
// Get Connection Success status
subscribe_nc.onStatus = function(ncInfoObj:Object):Void  {
	
	if(subDebugState)
		debugtxt(ncInfoObj.code);
	
	if (ncInfoObj.code == "NetConnection.Connect.Success") {
		Connect_Sub_btn.setLabel("Close");
		Subscribe_btn.enabled = true;
	}
};

////////////////////////////////////////////////////
// FCSubscribe callback
subscribe_nc.onFCSubscribe = function(info)
{
	
	debugtxt("[*** onFCSubscribe ***]");
	
	if (info.code == "NetStream.Play.Start")
	{
		subscribe_ns = new NetStream(subscribe_nc);
		
		Subscribe_Video.attachVideo(subscribe_ns);
		Subscribe_Video.attachAudio(subscribe_ns);
		
		//always a good practice to set the NS buffer (seconds)
		subscribe_ns.setBufferTime(1);
		
		////subscriber netstream time
		var debug2_interval:Number = setInterval(debug2Thing, 100, subscribe_ns);
		
		subscribe_ns.play(streamName);
		
		subscribe_ns.onStatus = function( info )
		{
			if(subDebugState)
				debugtxt (info.code + ", " + info.description);
		}
	}
	else
	{
		if(subDebugState)
			debugtxt (info.code + ", " + info.description);
	}
}

////////////////////////////////////////////////////
// onFCUnsubscribe callback
subscribe_nc.onFCUnsubscribe = function(info)
{
	debugtxt("[*** onFCUnsubscribe ***]");
}

///////////////////////////////////////////////////////
// debug texts
function debug1Thing(that_ns:NetStream) {
    debug1_txt.text = "time: "+Math.floor(that_ns.time); 
	cam_fps.text = client_cam.currentFps;
}
function debug2Thing(that_ns:NetStream) {
    debug2_txt.text = "time: "+Math.floor(that_ns.time);
}
	
///////////////////////////////////////////////////
// mic audio level
var activityLevel_pb:mx.controls.ProgressBar;
activityLevel_pb.mode = "manual";
activityLevel_pb.label = "Mic check: %3%%";
activityLevel_pb.setStyle("themeColor", "0xFF0000");

////////////////////////////////////////////////////
// Audio level montoring
this.onEnterFrame = function() {
    activityLevel_pb.setProgress(client_mic.activityLevel, 100);
};

client_mic.onActivity = function(active:Boolean) {
    if (active) {
    	var haloTheme_str:String = "haloGreen";
    } else {
    	var haloTheme_str:String = "0xFF0000";
    }
    activityLevel_pb.setStyle("themeColor", haloTheme_str);
};

///////////////////////////////////////////////////
// Audio Codec type 
audioCodecListener.click = function(eventObj:Object)
{
	for(i in eventObj){
		debugtxt(i + ": " + eventObj[i]);
	}
	codecType = eventObj.target.data;
	
	setCodecType(codecType);
}

///////////////////////////////////////////////////
// Sets Audio Codec type
function setCodecType(type:Number):Void
{
	
	switch(type){
		case kAudioNM:
			debugtxt("NellyMoser");
			rad_codec_nellymoser.selected = true;
			rad_codec_speex.selected = false;
			client_mic.setCodec("NellyMoser");
			rad_44.enabled = true;
			rad_22.enabled = true;
			rad_16.enabled = true;
			rad_11.enabled = true;
			rad_8.enabled = true;
			rad_5.enabled = true;
		break;
		case kAudioSpeex:
			debugtxt("Speex");
			rad_codec_speex.selected = true;
			rad_codec_nellymoser.selected = false;
			client_mic.setCodec("Speex");
			rad_44.enabled = false;
			rad_22.enabled = false;
			rad_16.enabled = true;
			rad_11.enabled = false;
			rad_8.enabled = false;
			rad_5.enabled = false;
			rad_16.selected = true;
		break;
		}
}

////////////////////////////////////////////////////
// Audio SampleRate radio buttons
// NM = all, speex = 16K only
sampleRateListener.click = function(eventObj:Object)
{
	var sr = eventObj.target.data;
	
	rad_44.selected = false;
	rad_22.selected = false;
	rad_16.selected = false;
	rad_11.selected = false;
	rad_8.selected = false;
	rad_5.selected = false;
	
	client_mic.setRate(sampleRate_array[sr]);
	
	switch(sr){
		case 5:
			rad_44.selected = true;
		break;
		case 4:
			rad_22.selected = true;
		break;
		case 3:
			rad_16.selected = true;
		break;
		case 2:
			rad_11.selected = true;
		break;
		case 1:
			rad_8.selected = true;
		break;
		case 0:
			rad_5.selected = true;
		break;
	}
		debugtxt("sampleRate= " + sampleRate_array[sr]);
}

//////////////////////////////////////
// general debug text area
function debugtxt(txt:String)
{
	debug_ta.text += txt;
	debug_ta.text += newline;
	debug_ta.vPosition = debug_ta.maxVPosition;
}